@@ -22,24 +22,8 @@ phase3 <- function(z, x, ...)
     z$returnDeps <- z$returnDepsStored
 
     if (x$checktime) z$ctime <- proc.time()[3]
+
     ## fix up iteration numbers if using multiple processors
-   # if (10 %% int == 0)
-   # {
-   #     firstNit <- 10
-   # }
-  #  else
-   # {
-   #     firstNit <- 10 + int - 10 %% int
-   # }
-   # if ((x$n3 - firstNit) %% int == 0)
-   # {
-   #     endNit <- x$n3
-   # }
-   # else
-   # {
-   #     endNit <- x$n3  + int - (x$n3 - firstNit) %% int
-   # }
-   #  cat("endNit", endNit, "n3 ", x$n3, "\n")
     if (x$n3 %% int > 0)
     {
         endNit <- x$n3 + int - x$n3 %%int
@@ -48,17 +32,9 @@ phase3 <- function(z, x, ...)
     {
         endNit <- x$n3
     }
-   z$n3 <- endNit
-    z$sf <- matrix(0, nrow = z$n3, ncol = z$pp)
-    z$sf2 <- array(0, dim = c(z$n3, f$observations - 1, z$pp))
-    z$accepts3 <- matrix(0, nrow=z$n3, ncol=7)
-    z$rejects3 <- matrix(0, nrow=z$n3, ncol=7)
-    z$aborted3 <- matrix(0, nrow=z$n3, ncol=7)
-    if (!is.null(z$cconditional) && z$cconditional)
-    {
-        z$ntim <- matrix(NA, nrow=z$n3, ncol=f$observations - 1)
-    }
-    z$sims <- vector("list", z$n3)
+	z$n3 <- endNit
+    z$Phase3nits <- endNit
+
     ## revert to original requested method for phase 3 unless symmetric
     if (z$FinDiff.method && !x$FinDiff.method &&
         (!is.null(z$FinDiffBecauseSymmetric)) && z$FinDiffBecauseSymmetric)
@@ -77,15 +53,7 @@ phase3 <- function(z, x, ...)
     else
         Report('Estimation of derivatives by the LR method (type 1).\n\n', outf)
 
-    if (!x$maxlike & !z$FinDiff.method)
-    {
-        z$ssc <- array(0, dim = c(z$n3, f$observations - 1, z$pp))
-    }
-    else
-    {
-        z$sdf <- array(0, dim = c(z$n3, z$pp, z$pp))
-        z$sdf2 <- array(0, dim = c(z$n3, f$observations - 1, z$pp, z$pp))
-    }
+	z <- createSiena07stores(z, z$n3, f)
     xsmall<- NULL
     zsmall <- makeZsmall(z)
     if (!x$maxlike && !is.null(z$writefreq))
@@ -98,16 +66,6 @@ phase3 <- function(z, x, ...)
     }
     z <- AnnouncePhase(z, x)
     Report('Simulated values, phase 3.\n', cf)
-    #cat("endNit", endNit, "n3 ", x$n3, "\n")
-    #if (x$n3 %% int > 0)
-    #{
-    #    endNit <- x$n3 + int - x$n3 %%int
-    #}
-    #else
-    #{
-    #    endNit <- x$n3
-    #}
-    #cat("endNit", endNit, "n3", x$n3, "z$n3", z$n3, "\n")
     nits <- seq(1, endNit, int)
     nits11 <- min(c(endNit, nits[nits >= 11]))
     writefreq <- z$writefreq
@@ -115,223 +73,12 @@ phase3 <- function(z, x, ...)
     {
         z$writefreq <- 10
     }
-    for (nit in nits)
-    {
-        z$nit <- nit
-        if (is.null(writefreq))
-        {
-            if (nit == nits[2])
-            {
-                z$time1 <- proc.time()[[3]]
-            }
-            else if (nit == nits11)
-            {
-                time1 <- proc.time()[[3]] - z$time1
-                if (time1 > 1e-5)
-                {
-                    z$writefreq <- max(1, round(20.0 / time1))
-                }
-                else
-                {
-                    z$writefreq <- 20
-                }
-                if (is.batch())
-                {
-                    z$writefreq <-  z$writefreq * 2 ##compensation for it
-                    ## running faster with no tcl/tk
-                }
-                z$writefreq <- roundfreq(z$writefreq)
-                writefreq <- z$writefreq
-            }
-        }
-        if (nit <= 5 || nit == 10 || (int==1 && nit %% z$writefreq == 0 ) ||
-            (int > 1 && (z$writefreq + 1) < z$n3%/%int &&
-             nit %in% nits[seq(z$writefreq + 1, x$n3 %/% int,
-                                          z$writefreq)]))
-        {
-            if (!is.batch())
-            {
-                DisplayIteration(z)
-              #  DisplayTheta(z)
-                DisplayDeviations(z, z$sf[nit-1,])
-            }
-           ## else
-           ## {
-           ##     Report(c('Phase ', z$Phase,' Iteration ',nit,'\n'))
-           ## }
-            if (nit %% z$writefreq == 0 || (int > 1 &&
-                       nit %% z$writefreq == 1) )
-            {
-                increment <- ifelse(nit <= 5, int,
-                                    ifelse(nit <= 10, 5, z$writefreq * int))
-                val<- getProgressBar(z$pb)
-                if (z$FinDiff.method)
-                    val <- val + increment * (z$pp + 1)
-                else
-                    val <- val + increment
-                ## sort out progress bar
-                z$pb <- setProgressBar(z$pb, val)
-                if (is.batch())
-                    Report(c("Phase ", z$Phase, " Iteration ", nit,
-                             " Progress ",
-                             round(val / z$pb$pbmax * 100), "%\n"), sep='')
-            }
-        }
-###############################
-        z <- doPhase3it(z, x, nit, zsmall=zsmall, xsmall=xsmall, ...)
-##############################
-        ##  browser()
-        if (!is.batch())
-        {
-          #  if (nit<=3 || nit %%z$writefreq ==0)
-          #      DisplayTheta(z)
-            if (nit < 10)
-                Report(c("  ", nit, " ", format(z$sf[nit,], width=10,
-                                             digits=4, nsmall=4), "\n"), cf)
-            if (nit >= 10)
-            {
-                CheckBreaks()
-                ##    if (nit==10) set up stopkey hint Early termination of estimation
-                if( UserInterruptFlag())
-                {
-                    Report(c("The user asked for an early stop of the algorithm ",
-                             "during phase 3.\n"), outf)
-                    z$Phase3Interrupt <- TRUE
-                    if (nit < 500)
-                    {
-                        if (EarlyEndPhase2Flag())
-                            Report('This implies that ', outf)
-                        else
-                            Report('This implies that the estimates are as usual,\nbut ',
-                                   outf)
-                        Report(c('the diagnostic checks, covariance matrices and',
-                                 't-values \nare less reliable, because they are now',
-                                 'based on only', nit,
-                                 'phase-3 iterations.\n'), outf)
-                    }
-                    z$sf <- z$sf[1:nit, , drop=FALSE]
-                    z$sf2 <- z$sf2[1:nit, , , drop=FALSE]
-                    if (!x$maxlike)
-                    {
-                        z$ssc <- z$ssc[1:nit, , , drop=FALSE]
-                    }
-                    else
-                    {
-                        z$sdf <-z$sdf[1:nit, , , drop=FALSE]
-                        z$sdf2 <-z$sdf2[1:nit, , , ,drop=FALSE]
-                    }
-                    z$sims <-z$sims[1:nit]
-                    endNit <- nit
-                    break
-                }
-                if (UserRestartFlag())
-                    break
-            }
-        }
-    }
-    if (!z$OK || UserRestartFlag())
-    {
-        return(z)
-    }
-    z$Phase3nits <- endNit
-    z$n3 <- endNit
+	z <- doPhase1or3Iterations(3, z, x, zsmall, xsmall, nits, 0, nits11,
+							   writefreq)
     z <- phase3.2(z,x)
     z
 }
 
-##@doPhase3it siena07 Does one iteration in phase 3
-doPhase3it<- function(z, x, nit, zsmall, xsmall, ...)
-{
-    int <- z$int
-    if (int == 1)
-    {
-        zz <- x$FRAN(zsmall, xsmall)
-        if (!zz$OK)
-        {
-            z$OK <- zz$OK
-            z$zz <- zz
-            return(z)
-        }
-        z$n<- z$n + 1
-    }
-    else
-    {
-  ##zz <- clusterCall(cl, simstats0c, zsmall, xsmall)
-        zz <- clusterCall(z$cl, usesim, zsmall, xsmall)
-        z$n <- z$n + z$int
-      #  browser()
-   }
-    if (int == 1)
-    {
-        fra <- colSums(zz$fra)
-        fra <- fra - z$targets
-        z$sf[nit, ] <- fra
-        z$sf2[nit, , ] <- zz$fra
-        z$sims[[nit]] <- zz$sims
-        fra <- fra + z$targets
-        fra2 <- zz$fra
-    }
-    else
-    {
-        for (i in 1:int)
-        {
-            fra <- colSums(zz[[i]]$fra)
-            fra <- fra - z$targets
-            z$sf[nit + (i - 1), ] <- fra
-            z$sf2[nit + (i - 1), , ] <- zz[[i]]$fra
-            z$sims[[nit + (i - 1)]] <- zz[[i]]$sims
-        }
-        fra <- t(sapply(zz, function(x)colSums(x$fra)))
-        fra2 <- t(sapply(zz, function(x)x$fra))
-        dim(fra2) <- c(int, nrow(zz[[1]]$fra), z$pp)
-    }
-    if ((!x$maxlike) && z$cconditional)
-    {
-        if (int==1)
-        {
-            z$ntim[nit,] <- zz$ntim0
-        }
-        else
-        {
-            for (i in 1:int)
-            {
-                z$ntim[nit+(i-1),] <- zz[[i]]$ntim0
-            }
-        }
-    }
-    if (z$FinDiff.method)
-    {
-        z <- FiniteDifferences(z, x, fra, fra2, ...)
-        z$sdf[nit:(nit + (int - 1)), , ] <- z$sdf0
-        z$sdf2[nit:(nit + (int - 1)), , ,] <- z$sdf02
-    }
-    else if (x$maxlike) ## as far as I can see
-    {
-        z$sdf[nit, , ] <- zz$dff
-        z$sdf2[nit, , , ] <- zz$dff2
-        z$accepts3[z$nit, ] <- zz$accepts
-        z$rejects3[z$nit, ] <- zz$rejects
-        z$aborted3[z$nit, ] <- zz$aborts
-    }
-    else
-    {
-            if (int==1)
-            {
-                if (!is.null(zz[['sc']]))
-                    z$ssc[nit , ,] <- zz$sc
-            }
-            else
-            {
-                for (i in 1:int)
-                {
-                    if (!is.null(zz[[i]][['sc']]))
-                        z$ssc[nit + (i - 1), , ] <- zz[[i]]$sc
-                }
-            }
-        }
-    z
-}
-
 ##@phase3.2 siena07 Processing at end of phase 3
 phase3.2 <- function(z, x, ...)
 {
@@ -343,10 +90,11 @@ phase3.2 <- function(z, x, ...)
     z <- PotentialNR(z, x, FALSE)
     if (any(z$newfixed))
     {
-        Report('There was a problem in obtaining convergence)\n', outf)
-        Report(c('Therefore, the program decided tentatively to fix parameter(s)',
-               cat(c(1:z$pp)[z$newfixed]), '.\n'), outf)
-        Report(c('It may be better to start all over again, ',
+        Report('There was a problem in obtaining convergence.\n', outf)
+        Report(c('Therefore, the program decided tentatively to fix',
+				 'parameter(s)',
+               c(1:z$pp)[z$newfixed], '.\n'), outf)
+        Report(c('It may be better to start all over again,',
                  'with better initial values or a reduced model.\n',
                  '(Check that you entered the data properly!)\n'), outf)
     }
@@ -359,8 +107,8 @@ phase3.2 <- function(z, x, ...)
         Report(c('basic rate parameter',
                  c('', 's')[as.integer(z$f$observations > 2) + 1],
                  ' as well as \n'), sep='', outf)
-    Report(c('convergence diagnostics, covariance and derivative matrices based on ',
-             z$Phase3nits, ' iterations.\n\n'), sep='', outf)
+    Report(c('convergence diagnostics, covariance and derivative matrices ',
+			 'based on ', z$Phase3nits, ' iterations.\n\n'), sep='', outf)
     Report('Information for convergence diagnosis.\n', outf)
     Report(c('Averages, standard deviations, ',
            'and t-ratios for deviations from targets:\n'), sep='', outf)
@@ -419,10 +167,16 @@ phase3.2 <- function(z, x, ...)
     if (x$maxlike)
     {
         Report('Autocorrelations during phase 3 : \n', outf)
-        Report(paste(format(1:z$pp,width=3), '. ',
+        Report(paste(format(1:z$pp, width=3), '. ',
                      format(z$sfl, width=8, digits=4),
-                     '\n'), outf)
+                     '\n', collapse=""), outf)
         Report ('\n', outf)
+        Report('Autocorrelations during phase 3 : \n', cf)
+        Report(paste(format(1:z$pp, width=3), '. ',
+                     format(z$sfl, width=8, digits=4),
+                     '\n', collapse=""), cf)
+        Report ('\n', cf)
+
     }
     for (j in 1:z$pp)
         if (z$diver[j]) ### don't understand this condition, as AllFixed is true
@@ -443,7 +197,8 @@ phase3.2 <- function(z, x, ...)
         PrtOutMat(z$msf, cf)
         Report('\n', cf)
         dfrac <- z$dfra - z$msf
-     ##   dfrac[z$fixed[row(dfrac)]|z$fixed[col(dfrac)]]<- 0 a clever way to do it
+		## dfrac[z$fixed[row(dfrac)] | z$fixed[col(dfrac)]] <- 0
+		## a clever way to do it
         dfrac[z$fixed, ] <- 0
         dfrac[ ,z$fixed] <- 0
         diag(dfrac)[z$fixed] <- 1
@@ -454,32 +209,42 @@ phase3.2 <- function(z, x, ...)
         }
     }
     else
+	{
         cov <- z$dinv %*% z$msfc %*% t(z$dinv)
+	}
     error <- FALSE
     if (inherits(try(msfinv <- solve(z$msfc)), "try-error"))
     {
         Report('Covariance matrix not positive definite: \n', outf)
         if (any(z$fixed || any(z$newfixed)))
+		{
             Report(c('(This may be unimportant, and related to the fact\n',
                    'that some parameters are fixed.)\n'), outf)
+		}
         else
+		{
             Report(c('This may mean that the reported standard errors ',
                      'are invalid.\n'), outf)
+		}
         z$msfinv <- NULL
     }
     else
+	{
         z$msfinv <- msfinv
+	}
     if (!is.null(cov))
     {
-        z$diver <- (z$fixed | z$diver | diag(cov) <1e-9) & (!z$AllUserFixed)
-        cov[z$diver,] <- Root(diag(cov))* 33
-        ##not sure this does not use very small vals
-        cov[,z$diver] <- Root(diag(cov))* 33
-        diag(cov)[z$diver]<- 999
+        z$diver <- (z$fixed | z$diver | diag(cov) < 1e-9) & (!z$AllUserFixed)
+		## beware: recycling works for one direction but not the other
+        diag(cov)[z$diver] <- 99 * 99
+        cov[z$diver, ] <- rep(Root(diag(cov)), each=sum(z$diver)) * 33
+		diag(cov)[z$diver] <- 99 * 99
+		cov[, z$diver] <- rep(Root(diag(cov)), sum(z$diver)) * 33
+        diag(cov)[z$diver] <- 99 * 99
     }
     z$covtheta <- cov
-   # ans<-InstabilityAnalysis(z)
-   z
+	## ans<-InstabilityAnalysis(z)
+	z
 }
 
 ##@CalulateDerivative3 siena07 Calculates derivative at end of phase 3
@@ -488,7 +253,7 @@ CalculateDerivative3<- function(z,x)
     z$mnfra <- colMeans(z$sf)
     if (z$FinDiff.method || x$maxlike)
     {
-        dfra <- t(apply(z$sdf, c(2,3), mean))
+		dfra <- t(as.matrix(Reduce("+", z$sdf) / length(z$sdf)))
     }
    else
     {
@@ -509,8 +274,9 @@ CalculateDerivative3<- function(z,x)
     z$msf <- cov(z$sf)
     if (z$Phase3nits > 2)
     {
-       z$sfl <- apply(z$sf, 2, function(x)acf(x, plot=FALSE, lag.max=1)[[1]][[2]])
-   }
+        z$sfl <- apply(z$sf, 2,
+                       function(x)acf(x, plot=FALSE, lag.max=1)[[1]][[2]])
+    }
     z$dfra1 <- z$dfra
     z$dfra <- dfra
     z
@@ -563,25 +329,347 @@ PotentialNR <-function(z,x,MakeStep=FALSE)
     Report('         change     new value \n', cf)
     Report(c(paste('  ', format(1:z$pp, width=2), '. ',
                    format(round(-fchange, digits=6), width=12, nsmall=6),
-                   format(round(z$theta-fchange, 6), width=12, nsmall=6),
+                   format(round(z$theta - fchange, 6), width=12, nsmall=6),
                    sep='', collapse='\n'), '\n'), cf)
     if (MakeStep) ##currently not used
     {
-        Report(c('\nAt the end of phase ',z$phase,', parameter values are \n'),outf)
+        Report(c('\nAt the end of phase ', z$phase,
+				 ', parameter values are \n'), outf)
         Report(paste(1:z$pp,'. ',format(z$theta,width=18,digits=6)),outf)
-        Report(c('A full Quasi-Newton-Raphson step after phase 3\nwould add the ',
-                 'following numbers to the parameters:\n'),outf)
-        Report(paste(1:z$pp,'. ',format(-round(fchange,6), width=12)),outf)
-        Report('\n\n',outf)
-        if (z$SomeFixed) fchange[z$fixed] <- 0
+        Report(c('A full Quasi-Newton-Raphson step after phase 3\n',
+				 'would add the ',
+                 'following numbers to the parameters:\n'), outf)
+        Report(paste(1:z$pp, '. ', format(-round(fchange, 6), width=12)), outf)
+        Report('\n\n', outf)
+        if (z$SomeFixed)
+		{
+			fchange[z$fixed] <- 0
+		}
         ##check positivity
         if (z$anyposj)
         {
-            neg<-z$posj& fchange>= z$theta
-            fchange[neg]<- 0.5*z$theta[neg]
+            neg <- z$posj & fchange >= z$theta
+            fchange[neg] <- 0.5 * z$theta[neg]
             Report(c('Intervention 3.4.3: positivity restriction after phase 3',
-                     'coordinate(s)',neg,'.\n'),cf)
+                     'coordinate(s)', neg, '.\n'), cf)
         }
     }
     z
 }
+
+doPhase1or3Iterations <- function(phase, z, x, zsmall, xsmall, nits, nits6=0,
+								  nits11=0, writefreq)
+{
+	int <- z$int
+	for (nit in nits)
+	{
+		z$nit <- nit
+		## initial steps differ between phases
+		if (phase == 1)
+		{
+			if (length(nits) > 1 && nit == nits[2])
+			{
+				time1 <- proc.time()['elapsed']
+				if (x$checktime)
+				{
+					z$ctime <- time1
+				}
+			}
+			if (nit == nits6)
+			{
+				time1 <- proc.time()['elapsed']-time1
+				if (time1 > 1e-5)
+				{
+					z$writefreq <- round(20.0/time1)
+				}
+				else
+				{
+					z$writefreq <- 5
+				}
+				if (is.batch())
+				{
+					z$writefreq <- 10 * z$writefreq
+				}
+				z$writefreq <- roundfreq(z$writefreq)
+			}
+			if (nit > nits6 && is.null(z$ctime))
+			{
+				time1 <- proc.time()['elapsed']
+				if (x$checktime)
+				{
+					z$ctime <- time1
+				}
+			}
+			DisplayIteration(z)
+		}
+		else ## phase 3
+		{
+			if (is.null(writefreq))
+			{
+				if (nit == nits[2])
+				{
+					z$time1 <- proc.time()[[3]]
+				}
+				else if (nit == nits11)
+				{
+					time1 <- proc.time()[[3]] - z$time1
+					if (time1 > 1e-5)
+					{
+						z$writefreq <- max(1, round(20.0 / time1))
+					}
+					else
+					{
+						z$writefreq <- 20
+					}
+					if (is.batch())
+					{
+						z$writefreq <-  z$writefreq * 2 ##compensation for it
+						## running faster with no tcl/tk
+					}
+					z$writefreq <- roundfreq(z$writefreq)
+					writefreq <- z$writefreq
+				}
+			}
+			if (nit <= 5 || nit == 10 || (int==1 && nit %% z$writefreq == 0 ) ||
+				(int > 1 && (z$writefreq + 1) < z$n3%/%int &&
+				 nit %in% nits[seq(z$writefreq + 1, x$n3 %/% int,
+								   z$writefreq)]))
+			{
+				if (!is.batch())
+				{
+					DisplayIteration(z)
+					##  DisplayTheta(z)
+					DisplayDeviations(z, z$sf[nit-1,])
+				}
+				## else
+				## {
+				##     Report(c('Phase ', z$Phase,' Iteration ',nit,'\n'))
+				## }
+				if (nit %% z$writefreq == 0 || (int > 1 &&
+						   nit %% z$writefreq == 1))
+				{
+					increment <- ifelse(nit <= 5, int,
+										ifelse(nit <= 10, 5, z$writefreq * int))
+					val<- getProgressBar(z$pb)
+					if (z$FinDiff.method)
+						val <- val + increment * (z$pp + 1)
+					else
+						val <- val + increment
+					## sort out progress bar
+					z$pb <- setProgressBar(z$pb, val)
+					if (is.batch())
+						Report(c("Phase ", z$Phase, " Iteration ", nit,
+								 " Progress ",
+								 round(val / z$pb$pbmax * 100), "%\n"), sep='')
+				}
+			}
+		}
+		## iteration proper
+		if (z$int == 1)
+		{
+			zz <- x$FRAN(zsmall, xsmall)
+			if (!zz$OK)
+			{
+				z$OK <- zz$OK
+				z$zz <- zz
+				return(z)
+			}
+			z$n <- z$n + 1
+			if (phase == 1)
+			{
+				z$phase1Its <- z$phase1Its + 1
+			}
+		}
+		else ### only do parallels at this level for forward simulation
+		{
+			zz <- clusterCall(z$cl, simstats0c, zsmall, xsmall)
+			z$n <- z$n + z$int
+			if (phase == 1)
+			{
+				z$phase1Its <- z$phase1Its + z$int
+			}
+		}
+		## now update the stores
+		if (z$int == 1)
+		{
+			fra <- colSums(zz$fra)
+			fra <- fra - z$targets
+			fra2 <- zz$fra
+			z$sf[z$nit, ] <- fra
+			z$sf2[z$nit, , ] <- zz$fra
+			z$sims[[z$nit]] <- zz$sims
+			z$chain[[z$nit]] <- zz$chain
+			fra <- fra + z$targets
+		}
+		else
+		{
+			for (i in 1:int)
+			{
+				fra <- colSums(zz[[i]]$fra)
+				fra <- fra - z$targets
+				z$sf[z$nit + (i - 1), ] <- fra
+				z$sf2[z$nit + (i - 1), , ] <- zz[[i]]$fra
+				z$sims[[z$nit + (i - 1)]] <- zz[[i]]$sims
+			}
+			fra2 <- t(sapply(zz, function(x)x$fra))
+			dim(fra2) <- c(int, nrow(zz[[1]]$fra), z$pp)
+			fra <- t(sapply(zz, function(x) colSums(x$fra)))
+		}
+		if (x$maxlike)
+		{
+			z$sdf[[z$nit]] <- zz$dff
+			z$sdf2[[z$nit]] <- zz$dff2
+			z$accepts[z$nit, , ] <- zz$accepts
+			z$rejects[z$nit, , ] <- zz$rejects
+			z$aborts[z$nit, , ] <- zz$aborts
+		}
+		else if (z$FinDiff.method)
+		{
+			z <- FiniteDifferences(z, x, fra, fra2)
+			for (i in 0:(z$int - 1))
+			{
+				z$sdf[[z$nit + i]] <- z$sdf0[i + 1, , ]
+				if (z$byWave)
+				{
+					z$sdf2[[z$nit + i]] <- z$sdf02[i + 1, , , ]
+				}
+			}
+		}
+		else
+		{
+			if (int==1)
+			{
+				if (!is.null(zz[['sc']]))
+				{
+					z$ssc[z$nit , ,] <- zz$sc
+				}
+			}
+			else
+			{
+                for (i in 1:int)
+                {
+                    if (!is.null(zz[[i]][['sc']]))
+					{
+                        z$ssc[z$nit + (i - 1), , ] <- zz[[i]]$sc
+					}
+                }
+            }
+		}
+		if ((!x$maxlike) && z$cconditional && z$Phase == 3)
+		{
+			if (int == 1)
+			{
+				z$ntim[z$nit, ] <- zz$ntim0
+			}
+			else
+			{
+				for (i in 1:int)
+				{
+					z$ntim[z$nit + (i-1), ] <- zz[[i]]$ntim0
+				}
+			}
+		}
+		## post processing also differs
+		if (phase == 1)
+		{
+			CheckBreaks()
+			if (UserInterruptFlag() || UserRestartFlag())
+			{
+				return(z)
+			}
+			val <- getProgressBar(z$pb)
+			if (z$FinDiff.method)
+			{
+				val <- val + z$pp + 1
+			}
+			else
+			{
+				val <- val + 1
+			}
+			z$pb <- setProgressBar(z$pb, val)
+			progress <- val / z$pb$pbmax * 100
+			if (z$nit <= 5 || z$nit %% z$writefreq == 0 || z$nit %%5 == 0 ||
+				x$maxlike || z$FinDiff.method ||
+				(z$int > 1 && z$nit %% z$writefreq < z$int))
+			{
+				##  Report(c('Phase', z$Phase, 'Iteration ', z$nit, '\n'))
+				if (is.batch())
+				{
+					Report(c('Phase ', z$Phase, ' Iteration ', z$nit,
+							 ' Progress: ', round(progress), '%\n'), sep='')
+				}
+				else
+				{
+					DisplayTheta(z)
+					DisplayDeviations(z, z$sf[z$nit, ])
+				}
+			}
+			if (!z$OK || UserRestartFlag() || UserInterruptFlag())
+				return(z)
+		}
+		else ## phase 3
+		{
+			if (!is.batch())
+			{
+				if (nit < 10)
+				{
+					Report(c("  ", nit, " ",
+							 format(z$sf[nit,], width=1, digits=4, nsmall=4),
+							 "\n"), cf)
+				}
+				if (nit >= 10)
+				{
+					CheckBreaks()
+					##    if (nit==10) set up stopkey hint
+					##   Early termination of estimation
+					if( UserInterruptFlag())
+					{
+						Report(c("The user asked for an early stop of the ",
+								 "algorithm during phase 3.\n"), outf)
+						z$Phase3Interrupt <- TRUE
+						if (nit < 500)
+						{
+							if (EarlyEndPhase2Flag())
+							{
+								Report('This implies that ', outf)
+							}
+							else
+							{
+								Report(c("This implies that the estimates",
+										 "are as usual,\nbut the "), outf)
+							}
+							Report(c("diagnostic checks, covariance",
+									 "matrices and t-values \nare less ",
+									 "reliable, because they ",
+									 'are now based on only', nit,
+									 'phase-3 iterations.\n\n'), outf)
+						}
+						z$sf <- z$sf[1:nit, , drop=FALSE]
+						z$sf2 <- z$sf2[1:nit, , , drop=FALSE]
+						if (!x$maxlike)
+						{
+							z$ssc <- z$ssc[1:nit, , , drop=FALSE]
+						}
+						else
+						{
+							z$sdf <-z$sdf[1:nit]
+							z$sdf2 <-z$sdf2[1:nit]
+						}
+						z$sims <-z$sims[1:nit]
+						z$Phase3nits <- nit
+						z$n3 <- nit
+						break
+					}
+					if (UserRestartFlag())
+						break
+				}
+			}
+		}
+		if (!z$OK || UserRestartFlag())
+		{
+			return(z)
+		}
+	}
+	z
+}
+
