@@ -34,7 +34,6 @@ Model::Model()
 	this->lneedChain = false;
 	this->lneedScores = false;
 	this->lneedDerivatives = false;
-	this->lneedChangeContributions = false;
 	this->lparallelRun = false;
 	this->linsertDiagonalProbability = 0;
 	this->lcancelDiagonalProbability = 0;
@@ -167,22 +166,6 @@ bool Model::needDerivatives() const
 {
 	return this->lneedDerivatives;
 }
-/**
- * Stores if change contributions are to be stored on ministeps
- */
-void Model::needChangeContributions(bool flag)
-{
-	this->lneedChangeContributions = flag;
-}
-
-
-/**
- * Returns if change contributions are to be stroed on ministeps
- */
-bool Model::needChangeContributions() const
-{
-	return this->lneedChangeContributions;
-}
 
 /**
  * Stores if this is a parallel run
@@ -192,7 +175,6 @@ void Model::parallelRun(bool flag)
 	this->lparallelRun = flag;
 }
 
-
 /**
  * Returns if this is a parallel run
  */
@@ -269,6 +251,34 @@ double Model::initialPermutationLength() const
 	return this->linitialPermutationLength;
 }
 
+/**
+ * Returns the current permutation length
+ */
+double Model::currentPermutationLength(int period) const
+{
+	return this->lcurrentPermutationLength[period];
+}
+
+/**
+ * Initializes the current permutation length
+ */
+void Model::initializeCurrentPermutationLength()
+{
+	for (int i = 0; i < this->lnumberOfPeriods; i++)
+	{
+		this->lcurrentPermutationLength.
+			push_back(this->linitialPermutationLength);
+	}
+}
+
+/**
+ * Stores the current permutation length
+ */
+void Model::currentPermutationLength(int period, double value)
+{
+	this->lcurrentPermutationLength[period] = value;
+}
+
 // ----------------------------------------------------------------------------
 // Section: Effect management
 // ----------------------------------------------------------------------------
@@ -595,40 +605,33 @@ vector<Chain *> & Model::rChainStore(int periodFromStart)
 	return this->lchainStore[periodFromStart];
 }
 
-void Model::partClearChainStore()
+void Model::clearChainStore(int keep, int groupPeriod)
 {
-	for (unsigned i = 0; i < this->lchainStore.size(); i++)
+	int size = this->lchainStore[groupPeriod].size();
+	if (keep > 0)
 	{
-		// need to keep the final one to continue from
-		for (unsigned j = 0; j < this->lchainStore[i].size() - 1; j++)
+		vector<Chain *>::iterator iter = this->lchainStore[groupPeriod].begin();
+
+		for (int j = 0; j < size - keep; j++)
 		{
 			//delete the chain pointed to
-			delete this->lchainStore[i][j];
-		}
-		vector<Chain *>::iterator iter = this->lchainStore[i].begin();
-		for (unsigned j = 0; j < this->lchainStore[i].size() - 1; j++)
-		{
-			//erase the entry
-			this->lchainStore[i].erase(iter);
-			iter++;
+			delete this->lchainStore[groupPeriod][j];
 		}
+		this->lchainStore[groupPeriod].erase(iter, iter + size - keep);
 
 	}
-}
-void Model::clearChainStore()
-{
-	for (unsigned i = 0; i < this->lchainStore.size(); i++)
+	else
 	{
-		deallocateVector(this->lchainStore[i]);
+		for (int chain = 0; chain < size; chain++)
+		{
+			delete lchainStore[groupPeriod][chain];
+		}
+		this->lchainStore[groupPeriod].clear();
+
 	}
 
 }
 
-void Model::clearChainStore(int periodFromStart)
-{
-	deallocateVector(this->lchainStore[periodFromStart]);
-}
-
 void Model::setupChainStore(int numberPeriods)
 {
 	this->lchainStore.resize(numberPeriods);
@@ -636,6 +639,7 @@ void Model::setupChainStore(int numberPeriods)
 
 void Model::deleteLastChainStore(int periodFromStart)
 {
+	delete this->lchainStore[periodFromStart].back();
 	this->lchainStore[periodFromStart].pop_back();
 }
 
