@@ -57,24 +57,19 @@ double OutdegreeEffect::egoStatistic(int ego, double * currentValues)
 
 
 /**
- * Returns the statistic corresponding to this effect as part of
+ * Returns the statistic corresponding to the given ego as part of
  * the endowment function with respect to the initial values of a
  * behavior variable and the current values.
  */
-double OutdegreeEffect::endowmentStatistic(const int * difference,
+double OutdegreeEffect::egoEndowmentStatistic(int ego, const int * difference,
 	double * currentValues)
 {
 	double statistic = 0;
-	int n = this->n();
 
-	for (int i = 0; i < n; i++)
+	if (difference[ego] > 0)
 	{
-		if (difference[i] > 0)
-		{
-			statistic += currentValues[i] * this->pNetwork()->outDegree(i);
-				//		-(currentValues[i] + difference[i]) *
-				//	this->pNetworkVariable()->pPredictorNetwork()->outDegree(i);
-		}
+//		statistic += currentValues[ego] * this->pNetwork()->outDegree(ego);
+		statistic -= difference[ego] * this->pNetwork()->outDegree(ego);
 	}
 
 	return statistic;
